<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DTO\Response\Message;

class FechamentoDiaForSale  extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $fechamentoDiaForSaleService;
    protected $util;
    
    public function __construct (Manager $dataSourceManager, Services\FechamentoDiaForSaleService $fechamentoDiaForSaleService, Utilities\Util $util){
        $this->fechamentoDiaService = $fechamentoDiaForSaleService;
        $this->util = $util;
    }
    
    public function findDataProcessamento(DTO\Request\Filter $request,DTO\Response $response){
       $conditions = $this->util->getParamsAsArray($request);
       $retorno = $this->fechamentoDiaService->findDataProcessamento($conditions['NRORG'], $conditions['CDFILIAL']);
       $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd5800_fecharDia_forSale', $retorno));
       
        
    }
    
    public function fechaDiaForSale(DTO\Request\DataSet $request, DTO\Response $response){
        $conditions = $request->getDataSet()->getRows()[0];
        $PesqMapaECF_PesqCxEcf = $this->fechamentoDiaService->validate_PesqMapaECF_PesqCxEcf($conditions['NRORG'], $conditions['CDFILIAL'], $conditions['DTPROCVND']);
        if($PesqMapaECF_PesqCxEcf){
            $response->addMessage(new Message("O Mapa Resumo ECF da filial não foi gerado para a data de processamento.", Message::TYPE_ERROR));
        } else {
            $Dtmovturcaix = $this->fechamentoDiaService->validate_Dtmovturcaix($conditions['NRORG'], $conditions['CDFILIAL'], $conditions['DTPROCVND']);
            if($Dtmovturcaix){
                $response->addMessage(new Message("Existem caixas abertos para esta data.", Message::TYPE_ERROR));
            } else {
                $Notas = $this->fechamentoDiaService->validate_notas($conditions['NRORG'], $conditions['CDFILIAL'], $conditions['DTPROCVND']);
                if($Notas){
                    $response->addMessage(new Message("Existem vendas NFCe concluídas no sistema que não foram aceitas na SEFAZ. Favor realizar a transmissão dessas notas.", Message::TYPE_ERROR));
                } else {
                    $PesqContole = $this->fechamentoDiaService->validate_PesqContole($conditions['NRORG'], $conditions['CDFILIAL'], $conditions['DTPROCVND']);
                    if($PesqContole == 2){
                        $response->addMessage(new Message("Dia para Retirada do Malote onde o valor de saída em Dinheiro não é igual ao valor do saldo anterior no caixa Controle. Entrar em contato com a Auditoria.", Message::TYPE_ERROR));
                    } else if ($PesqContole == 3){
                        $response->addMessage(new Message("Dia para Retirada do Malote sem operação de Saída no caixa Controle.", Message::TYPE_ERROR));
                    } else {
                        // Fim das Validações. Ponto de incio ao fechamento do dia 
                        $retorno = $this->fechamentoDiaService->fechaDiaForSale($conditions['NRORG'], $conditions['CDFILIAL'], $conditions['DTPROCVND'], $conditions['PROXDTPROCVND']);
                        if($retorno){
                           $response->addMessage(new Message("O fechamento do dia foi realizado com sucesso.")); 
                        }
                    }
                }
            }
        }
    }

}

//