<?php
namespace Teknisa\MAN\Services;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class GeraNotaVendaService {

    protected $entityManager;
    protected $paramsDelete = array();
    public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
        $this->entityManager = $entityManager;
    }
     
    public function getVendas($conditions) {
        
        $params = array(
         	":NRORG"        => $conditions['NRORG'],
         	":CDFILIAL"     => $conditions['CDFILIAL'],
         	":CDCONSUMIDOR" => ($conditions['CDCONSUMIDOR'] == "" ? null : (empty($conditions['CDCONSUMIDOR']) ? null : implode('_', $conditions['CDCONSUMIDOR']))),
         	":CDCAIXA"      => is_array($conditions['CDCAIXA']) ? implode('_',$conditions['CDCAIXA']) : null,
         	":CDLOJA"       => $conditions['CDLOJA'],
         	":CDCLIENTE"    => $conditions['CDCLIENTE'],
         	":DTINICIAL"    => $conditions['DTINICIAL'],
         	":DTFINAL"      => $conditions['DTFINAL'],
         	":VENDACONS"    => $conditions['VENDACONS'],
         	":CDTIPORECE"   => $conditions['CDTIPORECE'],
         	":ALLFIEDLS"    => 'T'
 		);
 		
 		 if(!empty($conditions)){
            foreach($conditions as $key => $value){
                if(strpos($key, '|')){
                    $params[':ALLFIEDLS'] = $value[0]['_ALL'];
                }
            }
        }
 		return $this->entityManager->getConnection()->fetchAll(self::QUERY_VENDAS, $params);
    }
    
    
    public function prepareProcedure($conditions) {
        if(isset($conditions['DTENTRVENDA'])) {
            $DTINICIAL = $conditions['DTENTRVENDA']['start'];
            $DTFINAL = $conditions['DTENTRVENDA']['end'];
        } else {
            $DTINICIAL = $DTFINAL = NULL;
        }
        
        if(!isset($conditions['CDCAIXA'])) {
            $CDCAIXA = null;
        } else {
            if(empty($conditions['CDCAIXA'])) {
                $CDCAIXA = null;
            } else {
                $CDCAIXA = implode('_', $conditions['CDCAIXA']);
            }
        }
        
        if(!isset($conditions['CHAVEVENDA'])) {
            $CHAVE = null;
        } else {
            if(empty($conditions['CHAVEVENDA'])) {
                $CHAVE = null;
            } else {
                $CHAVE = implode('_', $conditions['CHAVEVENDA']);
            }
        }
        
        if(!isset($conditions['CDCONSUMIDOR'])) {
            $CDCONSUMIDOR = null;
        } else {
            if(empty($conditions['CDCONSUMIDOR'])) {
                $CDCONSUMIDOR = null;
            } else {
                $CDCONSUMIDOR = implode('_', $conditions['CDCONSUMIDOR']);
            }
        }
        
        if(isset($conditions['DSENDECLIE'])) {
            $DSENDECLIE = substr($conditions['DSENDECLIE'], 0, 50);
        } else {
            $DSENDECLIE = NULL;
        }
        
        if($conditions['NMBAIRCLIE'] !== "") {
            $NMBAIRCLIE = substr($conditions['NMBAIRCLIE'], 0, 30);
        } else {
            $NMBAIRCLIE = NULL;
        }
        
        if($conditions['CDINSCESTANF'] !== "") {
            $NRINSESTDESNF = $conditions['CDINSCESTANF'];
        } else {
            $NRINSESTDESNF = NULL;
        }
        
        $DTSAIDA = $conditions['DTENTRSAIDNF'];
        $CDFILIAL = $conditions['CDFILIAL'];
        $CDLOJA = $conditions['CDLOJA'];
        $CDSERIE = $conditions['CDSERIE'];
        $NRINSJUDESNF = str_replace('/', '', str_replace('-', '', str_replace('.', '', $conditions['NRINSJURCLIE'])));
        $CDCLIENTE = !empty($conditions['CDCLIENTE']) ? $conditions['CDCLIENTE'] : $conditions['IDTPIJUDESNF'].$NRINSJUDESNF;
        $NMFANTCLIE = !empty($conditions['CDCLIENTE']) ? '' : $conditions['NMRAZSOCCLIE2'];
        $CDTIPOOPER = $conditions['CDTIPOOPER'];
        $P_CDFILIPROC = $conditions['P_CDFILIPROC'];
        $CDOPERADOR = $conditions['P_CDOPERADOR'];
        $P_DSMENSAGEM = $conditions['P_DSMENSAGEM'];
        $CDPAIS = $conditions['CDPAIS'];
        $SGESTADO = $conditions['SGESTADO'];
        $CDMUNICIPIO = $conditions['CDMUNICIPIO'];
        $IDTPIJUDESNF = $conditions['IDTPIJUDESNF'];
        $NMRAZSOCCLIE = $conditions['NMRAZSOCCLIE2'];
        $VENDACONS = $conditions['VENDACONS'];
        $CDCLIENTEVENDA = $conditions['CDCLIENTEVENDA'];
        //alterado a pedido da Pebbian para habilitar possibilidade de exclusao da Nota Gerada, deixando como default VND
        //$IDSISTEMA = $conditions['P_IDSISTEMA'];
        $IDSISTEMA = 'VND';
        $IDPRESENCIAL = isset($conditions['IDPRESENCIAL']) ? $conditions['IDPRESENCIAL'] : '9';
        $NRCEPVEN = !empty($conditions['NRCEPVEN']) ? $this->limpaCampo($conditions['NRCEPVEN']) : null;
        $this->geraNotaVendaSaida ($DTINICIAL, $DTFINAL, $CDCAIXA, $CHAVE, $NRCEPVEN, $DSENDECLIE, $NMBAIRCLIE, $DTSAIDA, $CDFILIAL, $CDLOJA, 
                                   $CDCLIENTE, $CDTIPOOPER, $P_CDFILIPROC, $CDOPERADOR, $P_DSMENSAGEM, $CDPAIS, $SGESTADO, $CDMUNICIPIO, $IDTPIJUDESNF, $NRINSJUDESNF, 
                                   $NMRAZSOCCLIE, $NRINSESTDESNF, $VENDACONS, $CDCLIENTEVENDA, $IDSISTEMA, $CDCONSUMIDOR, $CDSERIE, $IDPRESENCIAL, $NMFANTCLIE);
    }
    
    public function geraNotaVendaSaida ($DTINICIAL, $DTFINAL, $CDCAIXA, $CHAVE, $NRCEPVEN, $DSENDECLIE, $NMBAIRCLIE, $DTSAIDA, $CDFILIAL, $CDLOJA, 
                                       $CDCLIENTE, $CDTIPOOPER, $P_CDFILIPROC, $CDOPERADOR, $P_DSMENSAGEM, $CDPAIS, $SGESTADO, $CDMUNICIPIO, $IDTPIJUDESNF, $NRINSJUDESNF, 
                                       $NMRAZSOCCLIE, $NRINSESTDESNF, $VENDACONS, $CDCLIENTEVENDA, $IDSISTEMA, $CDCONSUMIDOR, $CDSERIE, $IDPRESENCIAL, $NMFANTCLIE) {
        $result = false;
        $P_CDCCUSCLIE = null;
        $P_CDTIPOCONS = null;
        $NMRAZSOCCLIE= substr($NMRAZSOCCLIE, 0, 50);
        
        try {
            $params[] = new Param('P_CDFILIAL'      ,   Param::PARAM_INPUT, $CDFILIAL , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDLOJA'        ,   Param::PARAM_INPUT, $CDLOJA , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDCAIXA'       ,   Param::PARAM_INPUT, $CDCAIXA , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_LISTAVENDA'    ,   Param::PARAM_INPUT, $CHAVE , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDPRODUTO'     ,   Param::PARAM_INPUT, NULL , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_DTINICIAL'     ,   Param::PARAM_INPUT, $DTINICIAL , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_DTFINAL'       ,   Param::PARAM_INPUT, $DTFINAL , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDCLIENTE'     ,   Param::PARAM_INPUT, $CDCLIENTE , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_NMFANTCLIE'    ,   Param::PARAM_INPUT, $NMFANTCLIE , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_DTSAIDA'       ,   Param::PARAM_INPUT, $DTSAIDA , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDSERIE'       ,   Param::PARAM_INPUT, $CDSERIE, Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDTIPOOPER'    ,   Param::PARAM_INPUT, $CDTIPOOPER , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDFILIPROC'    ,   Param::PARAM_INPUT, $CDFILIAL , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDOPERADOR'    ,   Param::PARAM_INPUT, $CDOPERADOR , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_DSMENSAGEM'    ,   Param::PARAM_INPUT, $P_DSMENSAGEM , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_IDTPIJUDESNF'  ,   Param::PARAM_INPUT, $IDTPIJUDESNF , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_NRINSJUDESNF'  ,   Param::PARAM_INPUT, $NRINSJUDESNF , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_NMRAZSODESNF'  ,   Param::PARAM_INPUT, $NMRAZSOCCLIE , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_DSENDEDESTNF'  ,   Param::PARAM_INPUT, $DSENDECLIE , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_NMBAIRDESTNF'  ,   Param::PARAM_INPUT, $NMBAIRCLIE , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDPAIS'        ,   Param::PARAM_INPUT, $CDPAIS , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_SGESTADO'      ,   Param::PARAM_INPUT, $SGESTADO , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDMUNICIPIO'   ,   Param::PARAM_INPUT, $CDMUNICIPIO , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_NRINSESTDESNF' ,   Param::PARAM_INPUT, $NRINSESTDESNF , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_VENDACONS'     ,   Param::PARAM_INPUT, $VENDACONS , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDCLIENVENDA'  ,   Param::PARAM_INPUT, $CDCLIENTEVENDA , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_IDSISTEMA'     ,   Param::PARAM_INPUT, $IDSISTEMA , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDCONSUMIDOR'  ,   Param::PARAM_INPUT, $CDCONSUMIDOR , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDCCUSCLIE'    ,   Param::PARAM_INPUT, $P_CDCCUSCLIE , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_CDTIPOCONS'    ,   Param::PARAM_INPUT, $P_CDTIPOCONS , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_IDPRESENCIAL'  ,   Param::PARAM_INPUT, $IDPRESENCIAL , Param::PARAM_TYPE_STR);
            $params[] = new Param('P_NRCEPDESTNF'   ,   Param::PARAM_INPUT, $NRCEPVEN , Param::PARAM_TYPE_STR);
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_NOTAFISCAL_VENDA', $params);
            $procedure->execute();   
            $result = true;   
        }
        
       catch (\Exception $e) {
            if(strpos($e->getMessage(), 'ORA-01400') == true){
                throw new \Exception ("Favor verificar se a vigência da inscrição estadual é válida");
			}
			else {
			  $primeiraOcorrencia = strpos($e->getMessage(), 'ORA-');
			  $mensagem = substr($e->getMessage(), $primeiraOcorrencia);
			  throw new \Exception ("Favor verificar se a filial está parametrizada para controlar estoque por almoxarifado, e se não tem almoxarifado para filial cadastrado. <br/>".$mensagem);
			}
       }
       
        return $result;
    }
    
    public function limpaCampo($valor){
        $valor = trim($valor);
        $valor = str_replace(".", "", $valor);
        $valor = str_replace(",", "", $valor);
        $valor = str_replace("-", "", $valor);
        $valor = str_replace("/", "", $valor);
        return $valor;
    }
    
    const QUERY_VENDAS = "SELECT * FROM (SELECT V.NRSEQVENDA, TO_CHAR(V.DTVENDA, 'DD/MM/YYYY HH24:MI:SS') AS DTVENDA, C.NMRAZSOCCLIE, V.CDCAIXA, NVL(V.NRCUPOMFIS, V.NRNOTAFISCALCE) AS CUPOM, V.NRINSCRCONS, CDSERIENFCE, 
               V.CDFILIAL||V.CDCAIXA||V.NRSEQVENDA AS CHAVEVENDA, CO.NMCONSUMIDOR, TO_CHAR(SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) + NVL(I.VRACRITVEND,0) - NVL(I.VRDESITVEND,0)),'FM99999D90') AS VRTOTVENDA, 
               V.CDFILIAL, V.CDLOJA, V.NRORG, V.CDCONSUMIDOR, V.CDCLIENTE, :VENDACONS AS VENDACONS, MC.CDTIPORECE
         FROM CLIENTE C, ITEMVENDA I, VENDA V 
         LEFT JOIN CONSUMIDOR CO ON V.CDCONSUMIDOR = CO.CDCONSUMIDOR AND CO.CDCLIENTE = :CDCLIENTE 
         JOIN (SELECT DISTINCT CDFILIAL, CDCAIXA, NRSEQVENDA, CDTIPORECE FROM MOVCAIXA MC WHERE CDFILIAL = :CDFILIAL
                        AND ((INSTR(:CDCAIXA, CDFILIAL||CDCAIXA) > 0) OR (:CDCAIXA IS NULL)) 
                        AND ((INSTR(:CDTIPORECE, CDTIPORECE) > 0) OR (:CDTIPORECE IS NULL)) ) MC ON V.CDFILIAL = MC.CDFILIAL AND V.CDCAIXA = MC.CDCAIXA AND V.NRSEQVENDA = MC.NRSEQVENDA
        WHERE (V.CDFILIAL = :CDFILIAL ) 
          AND (V.CDLOJA = :CDLOJA ) 
          AND (V.IDSITUVENDA = 'O' ) 
          AND (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND (V.CDCLIENTE = C.CDCLIENTE ) 
          AND ((:VENDACONS = 'N') OR (:VENDACONS = 'S' AND V.CDCONSUMIDOR IS NOT NULL)) 
          AND ((V.IDSTATUSNFCE IS NULL) OR (V.IDSTATUSNFCE = 'A')) 
          AND (I.CDFILIAL = V.CDFILIAL ) 
          AND (I.CDCAIXA = V.CDCAIXA ) AND (I.NRSEQVENDA = V.NRSEQVENDA) AND (I.NRLANCTONF IS NULL ) 
          AND ((V.CDCLIENTE = :CDCLIENTE) OR (:CDCLIENTE IS NULL)) 
          AND ((INSTR(:CDCAIXA, V.CDFILIAL||V.CDCAIXA) > 0) OR (:CDCAIXA IS NULL)) 
          AND ((INSTR(:CDCONSUMIDOR, V.CDCONSUMIDOR)>0) OR (:CDCONSUMIDOR IS NULL)) 
          AND V.NRORG = :NRORG
        GROUP BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.CDLOJA, V.DTVENDA, V.NRORG, V.CDCLIENTE, V.CDCONSUMIDOR, C.NMRAZSOCCLIE, V.NRCUPOMFIS, V.NRINSCRCONS, NRNOTAFISCALCE, CDSERIENFCE, CO.NMCONSUMIDOR, :VENDACONS, MC.CDTIPORECE
        ORDER BY 5, 3, 2)
        WHERE (NRSEQVENDA||DTVENDA||NMRAZSOCCLIE||CDCAIXA||CUPOM||NRINSCRCONS||CDSERIENFCE||VRTOTVENDA||NMCONSUMIDOR like :ALLFIEDLS  OR :ALLFIEDLS = 'T' )";
} 